<?php declare(strict_types=1);

namespace Weinklang\ProductImageBadge\Subscriber;

use Shopware\Core\Content\Media\MediaEntity;
use Shopware\Core\Content\Product\ProductEvents;
use Shopware\Core\Content\Product\SalesChannel\SalesChannelProductEntity;
use Shopware\Core\Framework\DataAbstractionLayer\Event\EntityLoadedEvent;
use Shopware\Core\Framework\Struct\ArrayStruct;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class ProductLoadedSubscriber implements EventSubscriberInterface
{
    public static function getSubscribedEvents(): array
    {
        return [
            ProductEvents::PRODUCT_LOADED_EVENT => 'onProductLoaded'
        ];
    }

    public function onProductLoaded(EntityLoadedEvent $productEntityEvent): void
    {
        // don't break the admin
        $source = $productEntityEvent->getContext()->getSource();
        if (method_exists($source, 'isAdmin') && $source->isAdmin()) {
            return;
        }

        /** @var SalesChannelProductEntity $product */
        foreach ($productEntityEvent->getEntities() as $product) {
            $media = $product->getMedia();
            if (null === $media) {
                // the product listing works with the cover and has no loaded media
                $productMediaEntity = $product->getCover();
                $mediaEntity = $productMediaEntity->getMedia();
                $this->addProductImageBadges($mediaEntity);
            } else {
                // the product details has loaded media
                $productMediaCollection = $product->getMedia();
                foreach ($productMediaCollection as $productMediaEntity) {
                    $mediaEntity = $productMediaEntity->getMedia();
                    $this->addProductImageBadges($mediaEntity);
                }
            }
        }
    }

    private function addProductImageBadges(MediaEntity $mediaEntity): void
    {
        $index = rand(0, 1);
        $badges = [
            'images/goldene-kammerpreismuenze.png',
            'images/silberne-kammerpreismuenze.png'
        ];
        unset($badges[$index]);

        $mediaEntity->addExtension('product_image_badges', new ArrayStruct(['badges' => $badges]));
    }
}
